#version 120

#define Shadows
#define ssr

const int RGBA16                 = 1;             
const int gcolorFormat           =  RGBA16;  
const int colortex5Format        =  RGBA16;
const int shadowMapResolution = 1024; //[1024 2048 4096 8192 16284]
const bool shadowHardwareFiltering = true;

const int maxf              = 10;				//number of refinements
const float stp             = 0.4;			//size of one step for raytracing algorithm
const float ref             = 1.0;			//refinement multiplier
const float inc             = 1.4;			//increasement factor at each step

uniform sampler2D gcolor;

uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferModelViewInverse;


uniform mat4 gbufferModelView;
uniform mat4 gbufferProjection;

uniform mat4 shadowModelView;

#include "/lib/poisson.glsl"

const int noiseTexResolution = 32;
const int noiseTextureResolution = 256; 

#define ShadowColor

const float shadowDistance = 128.0; //[32.0 64.0 128.0 256.0 512.0 1024.0]
const float shadowMapBias = 0.85;
const float sunPathRotation = -25.0;

uniform sampler2DShadow shadowtex0;
uniform sampler2DShadow shadowtex1;
uniform sampler2D shadowcolor0;
uniform sampler2D shadowcolor1;

uniform sampler2D depthtex1;
uniform sampler2D depthtex0;
uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D noisetex;
uniform sampler2D tex;
uniform sampler2D colortex2;
uniform sampler2D colortex3;


uniform int frameCounter;
uniform int isEyeInWater;
uniform int worldTime;

uniform float aspectRatio;
uniform float far;
uniform float frameTimeCounter;
uniform float near;
uniform float rainStrength;
uniform float timeAngle;
uniform float viewWidth;
uniform float viewHeight;

uniform vec3 skyColor;
uniform vec3 cameraPosition;
uniform vec3 shadowLightPosition;

uniform mat4 shadowProjection;

varying vec4 texcoord;
varying vec4 color;

#include "/lib/dither.glsl"

//Worldtime
float timefract = worldTime;

//Get the time of the day
float TimeSunrise  = ((clamp(timefract, 23000.0, 24000.0) - 23000.0) / 1000.0) + (1.0 - (clamp(timefract, 0.0, 4000.0)/4000.0));
float TimeNoon     = ((clamp(timefract, 0.0, 4000.0)) / 4000.0) - ((clamp(timefract, 8000.0, 12000.0) - 8000.0) / 4000.0);
float TimeSunset   = ((clamp(timefract, 8000.0, 12000.0) - 8000.0) / 4000.0) - ((clamp(timefract, 12000.0, 12750.0) - 12000.0) / 750.0);
float TimeMidnight = ((clamp(timefract, 12000.0, 12750.0) - 12000.0) / 750.0) - ((clamp(timefract, 23000.0, 24000.0) - 23000.0) / 1000.0);
vec2 wind[4] = vec2[4](vec2(abs(frameTimeCounter/1000.-0.5),abs(frameTimeCounter/1000.-0.5))+vec2(0.5),
					vec2(-abs(frameTimeCounter/1000.-0.5),abs(frameTimeCounter/1000.-0.5)),
					vec2(-abs(frameTimeCounter/1000.-0.5),-abs(frameTimeCounter/1000.-0.5)),
					vec2(abs(frameTimeCounter/1000.-0.5),-abs(frameTimeCounter/1000.-0.5)));

//Depth

//Get position of the camera in live
vec4 getCameraSpacePosition(in vec2 coord, float getDepth) {
    float depth = getDepth;
    vec4 positionNdcSpace = vec4(coord.s * 2.0 - 1.0, coord.t * 2.0 - 1.0, 2.0 * depth - 1.0, 1.0);
    vec4 positionCameraSpace = gbufferProjectionInverse * positionNdcSpace;

    return positionCameraSpace / positionCameraSpace.w;
}

//Get position of the world in live
vec4 getWorldSpacePosition(in vec2 coord, float getDepth) {
    vec4 positionCameraSpace = getCameraSpacePosition(coord, getDepth);
    vec4 positionWorldSpace = gbufferModelViewInverse * positionCameraSpace;
    positionWorldSpace.xyz += cameraPosition;

    return positionWorldSpace;
}

float ld(float depth) {
   return (2.0 * near) / (far + near - depth * (far - near));
}

vec3 nvec3(vec4 pos) {
    return pos.xyz/pos.w;
}

vec4 nvec4(vec3 pos) {
    return vec4(pos.xyz, 1.0);
}

float cdist(vec2 coord) {
	return max(abs(coord.s-0.5),abs(coord.t-0.5))*2.0;
}

#ifdef ssr
vec4 raytrace(vec3 fragpos, vec3 normal, float dither) {
    vec4 color = vec4(0.0);
	dither = fract(dither + frameTimeCounter);

    vec3 start = fragpos;
    vec3 rvector = normalize(reflect(fragpos, normal));
    vec3 vector = exp(stp) * rvector;
    vec3 oldpos = fragpos;
    fragpos += vector;
	vec3 tvector = vector;
    int sr = 0;
	float border = 0.0;
	vec3 pos = vec3(0.0);
    for(int i=0;i<5;i++){
        if (isEyeInWater > 0) break;
        pos = nvec3(gbufferProjection * nvec4(fragpos)) * 0.5 + 0.5;
		if (pos.x < 0 || pos.x > 1 || pos.y < 0 || pos.y > 1 || pos.z < 0 || pos.z > 1) break;
		float depth = texture2D(depthtex0,pos.xy).r;
		vec3 spos = vec3(pos.st, depth);
        spos = nvec3(gbufferProjectionInverse * nvec4(spos*2.0-1.0));
		if (abs(length(fragpos.xyz-spos.xyz)) < pow(length(vector)*pow(length(tvector),0.17),1.1)*1.1){

                sr++;
                if (sr >= maxf) break;
				tvector -=vector;
                vector *=ref;
		}
        vector *= inc;
        oldpos = fragpos;
        tvector += vector * (dither * 0.005 + 0.975);
		fragpos = start + tvector;
    }
	
	if (pos.z <1.0-1e-5){
		border = clamp(1.0 - pow(cdist(pos.st), 200.0), 0.0, 1.0);
		color.a = float(texture2D(depthtex0,pos.xy).r < 1.0);
		if (color.a > 0.5) color.rgb = texture2D(colortex0, pos.st).rgb;
		color.rgb = clamp(color.rgb,vec3(0.0),vec3(8.0));
		color.a *= border;

	}
	
    return color;
}
#endif

#define SHADOW_MAP_BIAS 0.85
vec4 BiasShadowProjection(in vec4 projectedShadowSpacePosition) {

	vec2 pos = abs(projectedShadowSpacePosition.xy * 1.165);
	vec2 posSQ = pos*pos;
	
	float dist = pow(posSQ.x*posSQ.x*posSQ.x + posSQ.y*posSQ.y*posSQ.y, 1.0 / 6.0);

	float distortFactor = (1.0 - SHADOW_MAP_BIAS) + dist * SHADOW_MAP_BIAS;

	projectedShadowSpacePosition.xy /= distortFactor*0.82;



	return projectedShadowSpacePosition;
}
//What gets the shadow position
vec3 getShadowSpacePosition(in vec2 coord, float getDepth) {
    vec4 positionWorldSpace = getWorldSpacePosition(coord, getDepth);

    positionWorldSpace.xyz -= cameraPosition;
    positionWorldSpace.xyz += mat3(gbufferModelViewInverse)*shadowLightPosition*0.007;
    vec4 positionShadowSpace = shadowModelView * positionWorldSpace;
    positionShadowSpace = shadowProjection * positionShadowSpace;
    positionShadowSpace = BiasShadowProjection(positionShadowSpace);

    return positionShadowSpace.xyz * 0.5 + 0.5;
}

mat2 getRotationMatrix(in vec2 coord) {
    float rotationAmount = texture2D(
        noisetex,
        coord * vec2(
            viewWidth / noiseTextureResolution,
            viewHeight / noiseTextureResolution
        )
    ).r;
    return mat2(
        cos(rotationAmount), -sin(rotationAmount),
        sin(rotationAmount), cos(rotationAmount)
    );
}

#define PCSS_SAMPLE_COUNT 2

//Lighting and shadow code
vec3 getShadowColor(in vec2 coord) {
    vec4 tempcol = vec4(0.0);
    float getDepth = texture2D(depthtex0, texcoord.st).r;
    vec3 shadowCoord = getShadowSpacePosition(coord, getDepth);
    vec3 shadowColor = vec3(0.0);
    mat2 rotationMatrix = getRotationMatrix(coord);
    for(int i = 0; i < samplePoints.length(); i++) {
            vec2 offset = samplePoints[i] / shadowMapResolution;
            offset = rotationMatrix * offset;
            float shadow0sample = shadow2D(shadowtex0, vec3(shadowCoord.xy+offset, shadowCoord.z)).r;
            float shadow1sample = shadow2D(shadowtex1, vec3(shadowCoord.xy+offset, shadowCoord.z)).r;
            shadowColor += shadow1sample;
            bool mask = abs(shadow1sample-shadow0sample)>0.1;         //this is better with a slight threshold
            if (mask) {
                if (isEyeInWater == 0) tempcol += texture2D(shadowcolor0, shadowCoord.xy+offset);
            } else tempcol += vec4(1.0, 1.0, 1.0, 0.0);
        
    }
    tempcol /= samplePoints.length();    
    vec3 shadowcole = mix(vec3(1.0), tempcol.rgb*tempcol.rgb*tempcol.rgb, sqrt(tempcol.a));
    shadowColor *= shadowcole;

    return vec3(shadowColor) / samplePoints.length();
    
}


vec3 calculateLitSurface(in vec3 color) {
    vec3 normalrgb = texture2D(colortex1, texcoord.st).xyz*2.0-1.0;
    float torchmap = texture2D(colortex2, texcoord.st).r;
    float skymap = texture2D(colortex2, texcoord.st).g;
    torchmap *= torchmap; skymap *= skymap;
    vec3 torchlight = pow(torchmap, 2.0)*2.8*vec3(0.8, 0.55, 0.4);
    vec3 ambientLighting = (vec3(0.2)*TimeSunrise + (vec3(0.54, 0.55, 0.95)*0.8)*TimeNoon + vec3(0.2)*TimeSunset + vec3(0.05)*TimeMidnight);
    vec3 sunLighting = (vec3(0.5, 0.5, 0.55)*TimeSunrise + vec3(1.0)*TimeNoon + vec3(0.5, 0.5, 0.55)*TimeSunset + vec3(0.00001)*TimeMidnight);
	
    vec3 sunlightAmount = getShadowColor(texcoord.st);
    float Diffuse = max(dot(normalrgb, normalize(shadowLightPosition)), 0.0);
    sunlightAmount *= vec3(min(Diffuse, float(sunlightAmount)));
    return color * ((ambientLighting+torchlight) + sunlightAmount*sunLighting);
}

#ifdef ssr
vec4 applyReflections(vec4 albedo, vec3 normalmap, vec3 fragpos, float dither) {
    float fresnel = pow(clamp(1.0 + dot(normalmap, fragpos.xyz),0.0,1.0),3.5);
    vec4 reflections = vec4(0.0);
    vec3 rvec = normalize(reflect(fragpos, normalmap));
    reflections = raytrace(fragpos, normalmap, dither);
    //reflections.rgb *= (0.8*TimeSunrise + 1.0*TimeNoon + 0.8*TimeSunset + 0.2*TimeMidnight);
    albedo.rgb = mix(albedo.rgb,reflections.rgb,fresnel * reflections.a);
	albedo.a = mix(albedo.a,1.0,fresnel);
    return albedo;
}
#endif

#define diagonal3(m) vec3((m)[0].x, (m)[1].y, m[2].z)
#define  projMAD(m, v) (diagonal3(m) * (v) + (m)[3].xyz)

vec3 getNDCSpacePos(vec3 pos){
	vec4 iProjDiag = vec4(gbufferProjectionInverse[0].x, gbufferProjectionInverse[1].y, gbufferProjectionInverse[2].zw);
    vec3 p3 = pos * 2. - 1.;
    vec4 fragpos = iProjDiag * p3.xyzz + gbufferProjectionInverse[3];
    return fragpos.xyz / fragpos.w;
}


void main() {
    vec3 fragpos = getNDCSpacePos(vec3(gl_FragCoord.xy/vec2(viewWidth,viewHeight),gl_FragCoord.z));
    float id = texture2D(colortex3, texcoord.st).a*255;
    vec4 albedo = texture2D(tex, texcoord.st)*color;
    float depth = texture2D(depthtex0, texcoord.st).r;
    bool isTerrain = depth<1.0;
    bool waterMask = id == 8 || id == 9;
    vec3 normalmRgb = texture2D(colortex1, texcoord.st).rgb*2.0-1.0;
    #ifdef ssr
    if (waterMask) albedo = applyReflections(albedo, normalmRgb, normalize(getCameraSpacePosition(texcoord.st, depth).xyz), bayer16(gl_FragCoord.xy));
    #endif

    #ifdef Shadows
    if (isTerrain) {
        albedo.rgb = calculateLitSurface(albedo.rgb);
    }
    #endif
/*DRAWBUFFERS:0*/
    gl_FragData[0] = albedo;

}